	PROGRAM BREMTHICK

C            VERSION 2              5 MARCH 2002
C
C    Address questions and comments to
C         Gordon D. Holman
C         NASA/Goddard Space Flight Center
C         Code 682
C         Greenbelt, MD 20771
C         Office phone: (301) 286-4636
C		E-Mail: holman@stars.gsfc.nasa.gov
C
C    Purpose -
C         This program computes the thick-target bremsstrahlung x-ray/gamma-ray  
C		spectrum from an isotropic electron distribution function provided in 
C		Subroutine DISTRN.  The units of the computed flux is photons per
C		second per keV per square centimeter.  
C
C    Files used - 
C		BREMCROSS.FOR		Bremsstrahlung cross section, differential in photon energy
C		BREMTHICK.FOR		This file
C		DMLINBR2.FOR		Gaussian Quadrature integration routines
C		FDBPLBR.FOR			Electron distribution function
C
C    Subroutines -                                                       
C         DISTRN     Electron distribution function
C		ELOSS	   Computes the electron energy loss rate (dE/dt)
C		FINNER	   Function - provides integrand for the inner integration
C         FTHICK	   Function - provides integrand for outer bremsstrahlung integration
C		BREMCROSS  Bremsstrahlung cross section
C		DMLINO	   Function - Gaussian Quadrature integration routine - outer integral
C		GAULEGO	   Computes abscissas and weights for the Gaussian Quadrature integration
C		DMLINI	   Function - Gaussian Quadrature integration routine - inner integral
C		GAULEGI	   Computes abscissas and weights for the Gaussian Quadrature integration
C
C    Reference - 
C         bremthickdoc.pdf
C
C    Revisions - 
C		Version 1: 5 March 2001
C		Version 2: 5 March 2002 - Expressions for FINNER and FTHICK corrected (PC replaced 
C									with BETA).  This removes two factors of GAMMA, 
C									correcting the photon spectrum at relativistic 
C									energies.  
  
      IMPLICIT NONE

      INTEGER*4   IERGQL, IERGQH, IERGQI, IPH, TIME0, ITIME, ITICK

	INTEGER*4	MAXFCN

	REAL*8		EELOW, EEBRK, EEHIGH, EMIN, EMAXL, EMAX

	REAL*8		GAMLOW, GAMHIGH, GAMBRK, P, Q

	REAL*8		DENSTY, AREA, Z

	REAL*8		RERR, DMLINO, FCOEFF, DECOEFF

	REAL*8		EPH, EEL, FLUXL, FLUXH, FLUX, FTHICK

	REAL*8		PI, MC2, CLIGHT, AU, R0

	PARAMETER	( PI = 3.14159265, MC2 = 510.98D+00 )

	PARAMETER	( CLIGHT = 2.9979D+10, AU = 1.496D+13 )

	PARAMETER	( R0 = 2.8179D-13 )

	COMMON		/DISTFCN/GAMLOW, GAMHIGH, GAMBRK, P, Q

	COMMON		/INTF/Z, EPH, RERR, EEBRK, IERGQI, MAXFCN

	EXTERNAL	FTHICK

C	*****************************************************************
	NAMELIST/input/AREA,DENSTY,Z,P,Q,EELOW,EEBRK,EEHIGH,
     1			   GAMLOW,GAMBRK,GAMHIGH,RERR
C	*****************************************************************
  
  	OPEN(unit=11,file="output.txt", status="REPLACE")
      OPEN(unit=12,file="error.txt", status="REPLACE")

      CALL SYSTEM_CLOCK(TIME0)

C			  ***Input parameters***

C				Electron distribution lower and upper cutoff energies
C				and break energy in keV.  

	EELOW	= 20.D+00
	EEBRK	= 200.D+00
	EEHIGH	= 1.0D+05

C				Double power law distribution spectral indices

	P		= 5.0D+00
	Q	    = 5.0D+00

C				Average atomic number (Z) and area impacted by the  
C				electron beam.  Nonthermal electron density (DENSTY).

	Z		= 1.2D0
	DENSTY	= 2.0D+07
	AREA	= 1.0D+18

C			  ***Specify relative error***

	RERR    = 0.001D+00

C			  ***Specify the maximum number of points for the Gaussian 
C				 Quadrature integration, MAXFCN.
C				 MAXFCN also determines the array sizes in Functions 
C				 DMLINO and DMLINI.

	MAXFCN = 2048

C				 Compute electron gammas corresponding to EELOW, EEBRK,
C				 and EEHIGH

	GAMLOW  = (EELOW/MC2) + 1.D0
	GAMBRK  = (EEBRK/MC2) + 1.D0
	GAMHIGH = (EEHIGH/MC2) + 1.D0

C				 Compute the numerical coefficient for the photon flux

	FCOEFF = (AREA/(4.D0*PI*AU**2))*(CLIGHT**2/MC2**2)*DENSTY

C				 Compute the numerical coefficient for the energy loss
C				 rate in keV/sec.  This should be multiplied by the 
C				 target plasma number density, but the number density
C				 has been left out because it cancels out of the 
C				 thick-target calculation.  

	DECOEFF = 4.D0*PI*R0**2*CLIGHT*MC2

C				 Loop over photon energy in keV (EPH)

	DO IPH = 0, 40

	  EPH = 10D0**(0.5D0 + 0.1D0*IPH)

C				 Initialize error flags

	  IERGQL = 0
	  IERGQH = 0
	  IERGQI = 0

C	             The integration is performed in two steps
C				 because of the discontinuity at GAMBRK.

	  IF( EPH .LT. EEBRK ) THEN

	    EMIN  = DMAX1(EELOW, EPH)

		EMAXL = EEBRK

	    FLUXL = DMLINO(FTHICK,EMIN,EMAXL,MAXFCN,RERR,IERGQL)

		EMAX = EEHIGH

	    FLUXH = DMLINO(FTHICK,EEBRK,EMAX,MAXFCN,RERR,IERGQH)

	    FLUX  = FCOEFF*(FLUXL + FLUXH)/DECOEFF

	  ELSE

		EMAX = EEHIGH

	    FLUX = DMLINO(FTHICK,EPH,EMAX,MAXFCN,RERR,IERGQH)
		FLUX = FCOEFF*FLUX/DECOEFF

	  END IF

C				 Write results to screen and files

	  WRITE(6,'(2ES9.2,3(3X,I10))') EPH, FLUX, IERGQL, IERGQH, IERGQI

	  WRITE(11,'(2ES14.4)')  EPH, FLUX

	  WRITE(12,'(ES14.4,3I5)') EPH, IERGQL, IERGQH, IERGQI

	END DO


      CALL SYSTEM_CLOCK(ITIME, ITICK)
      IF (ITICK /= 0)                                                  
     1WRITE (6,'(" Processing complete in ", F9.3, " seconds")')       
     2      REAL(ITIME-TIME0)/REAL(ITICK)

C	Write the NAMELIST parameters to the output file.
	WRITE(11,NML=input)
  
      STOP 'Bremsstrahlung Computations Finished'
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE ELOSS( EEL, DEDT )

C	Computes the energy dependent terms of the collisional energy loss 
C	rate for the energetic electrons, DEDT.  The numerical coefficient, 
C	DECOEFF, is computed in Program BREMTHICK.  

	IMPLICIT NONE

	REAL*8		EEL, DEDT, MC2, GAMMA, BETA

	PARAMETER	( MC2 = 510.98D+00 )

	GAMMA = (EEL/MC2) + 1.D0

	BETA = DSQRT( 1.D0 - 1.D0/GAMMA**2 )

	DEDT = DLOG( 6.9447D+09 * EEL ) / BETA

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	FUNCTION FINNER( EEL )

C	Returns the integrand for the inner integration

	IMPLICIT NONE

	REAL*8		EEL, FINNER, Z, EPH

	REAL*8		CROSS, GAMMA, BETA, DEDT, MC2

	PARAMETER	( MC2 = 510.98D+00 )

	COMMON		/INTF/Z, EPH

	GAMMA = (EEL/MC2) + 1.D0

	BETA = DSQRT( 1.D0 - 1.D0/GAMMA**2 )

	CALL BREMCROSS( Z, EEL, EPH, CROSS )

	CALL ELOSS(EEL, DEDT)

	FINNER = CROSS * BETA / DEDT

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	FUNCTION FTHICK( EO )

C	Returns the integrand for the (outer) bremsstrahlung integration.

	IMPLICIT NONE

	INTEGER*4	IERGQI, IERGQIL, IERGQIH, MAXFCN

	REAL*8		EO, FTHICK, Z, EPH, RERR, EEBRK

	REAL*8		GAMMA, BETA, CROSINT, CROSINTL, CROSINTH

	REAL*8		MC2, CLIGHT, FCN, FINNER, DMLINI

	PARAMETER	( MC2 = 510.98D+00, CLIGHT = 2.9979D+10 )

	COMMON		/INTF/Z, EPH, RERR, EEBRK, IERGQI, MAXFCN

	EXTERNAL FINNER

	GAMMA = (EO/MC2) + 1.D0

	BETA = DSQRT( 1.D0 - 1.D0/GAMMA**2 )

	CALL DISTRN( GAMMA, FCN )

C	Compute the value of the inner integral at energy EO.
C	Integrate in two parts if EEBRK is within the range 
C	of the integration.  

	IF( (EPH .LT. EEBRK) .AND. (EO .GT. EEBRK) ) THEN

	    CROSINTL = DMLINI(FINNER,EPH,EEBRK,MAXFCN,RERR,IERGQIL)

	    CROSINTH = DMLINI(FINNER,EEBRK,EO,MAXFCN,RERR,IERGQIH)

	    CROSINT  = CROSINTL + CROSINTH

		IERGQI = IERGQI + IERGQIL + IERGQIH

	ELSE

		CROSINT = DMLINI(FINNER,EPH,EO,MAXFCN,RERR,IERGQIH)

		IERGQI = IERGQI + IERGQIH

	END IF

	FTHICK = FCN * BETA * CROSINT

C	Replace the above line with the following to have DISTRN specify
C	the normalized electron flux density rather than the number density.
C	Specify the total electron flux density with the input parameter DENSTY.  

C	FTHICK = FCN * CROSINT / CLIGHT

	RETURN
	END